#include "ti83plus.inc"
#include "ion.inc"

.org progstart-2
.db t2ByteTok, tAsmCmp

;###############_CONSTANTS_##############
#DEFINE MAP_WIDTH 11        ;width of the map
#DEFINE MAP_HEIGHT 11       ;
#DEFINE COLUMNS_TO_DRAW (6+1)   ;columns to draw
#DEFINE HRZ_SCROLL 2       ;with 2 tiles between the player and the edge, scroll map
#DEFINE VERT_SCROLL 1

#DEFINE P_MONEY  0
#DEFINE P_X      2
#DEFINE P_Y      3
#DEFINE P_DIR    4
#DEFINE P_CHAR   5
#DEFINE P_SIZE   6

;###############_VARIABLES_################
chanceDeck = safeRAM4           ;0-16       SR4 HAS  66 BYTES
commChestDeck = chanceDeck+17   ;17-32
numPlayers  = commChestDeck+17  ;33
initialize:
    di
    ld hl,$89EC     ;byte after saferam1 (savesscreen) (creando nuestro "saferam")
    ld de,saferam3  ;guardar los valores aqui para no estropear todo
    ld bc,78
    ldir            ;guardarlo

    ld hl,chanceDeck+16
    call setUpCards ;create/shuffle chance cards
    ld hl,commChestDeck+16
    call setUpCards ;create/shuffle comm chest cards
    
    ld a,2
    ld (numPlayers),a
    ld hl,player1
    ld b,2
initPlayers:
    push bc
    ld de,1500
    ld (hl),e       ;money
    inc hl
    ld (hl),d
    inc hl
    ld (hl),10       ;playerX
    inc hl
    ld (hl),10       ;playerY
    inc hl
    ld (hl),0       ;direction
    inc hl
    ld b,256/4
    call randomNumber
    ld (hl),c       ;character's sprite#
    inc hl
    pop bc
    djnz initPlayers
    ld hl,currentPlayer
    ld (hl),0
    call loadPlayer
;main program loop
start:
    call drawMap
    call drawPlayer
    call drawGbuf
#include "monoKeyLoop.asm"
    jr start

;IMPORTANT! WE NEED TO RESTORE THE VALUES FROM BEFORE THE PROGRAM
quit:
    ld hl,saferam3  ;guardar los valores aqu? para no joderlo todo
    ld de,$89EC     ;byte after saferam1 (savesscreen)
    ld bc,78
    ldir            ;save it all
    ret

changePlayer:
    ld hl,currentPlayer
    inc (hl)
    ld a,(hl)
numberPlayers = $+1
    cp 0
    jr nz,loadPlayer
    ld (hl),0
loadPlayer:
    call getCurrentPlayer
    ld e,P_X
    add hl,de       ;hl points to player's x coord
    ld e,(hl)       ;pX
    inc hl
    ld d,(hl)       ;pY
    ld (playerX),de
    inc hl
    ld a,(hl)
    ld (playerDir),a
    ret

;#############################
;#GETCURRENTPLAYER
;#output: hl=address of current player
;#############################
getCurrentPlayer:
    ld hl,currentPlayer
    ld a,(hl)
    ld b,a
    add a,a
    add a,b
    add a,a         ;current player # x 6
    ld e,a
    ld d,0
    ld hl,players   ;hl points to current player
    ret
    
;########################
;#MAP MOVEMENT ROUTINES
;# displace the map
;########################
mapRight:
    ld hl,xCoord
    ld a,MAP_WIDTH-6
    cp (hl)
    ret z
    ld de,xOff
    ld a,(de)
    inc a
    inc a
    and $0F
    ld (de),a
    jp nz,updateRotation    ;if we have no overflow, we're all good.
    inc (hl)
    jp updateRotation

mapLeft:
    ld hl,xCoord
    xor a
    cp (hl)
    ld de,xOff
    ld a,(de)
    jr nz,mL_notLeft
    or a
    ret z
mL_notLeft:
    dec a
    dec a
    and $0F
    ld (de),a
    cp 14
    jp c,updateRotation ;if we have no overflow, we're all good.
    dec (hl)
    ld hl,mPL_adjustCP
    inc (hl)
    jp updateRotation

mapUp:
    ld hl,yCoord
    xor a
    cp (hl)
    ld de,yOff
    ld a,(de)
    jr nz,mU_notTop
    or a
    ret z
mU_notTop:
    dec a
    dec a
    and $0F
    ld (de),a
    cp 14
    ret c       ;if we have no overflow, we're all good.
    dec (hl)
    ld hl,mPU_adjustCP
    inc (hl)
    ret

mapDown:
    ld hl,yCoord
    ld a,MAP_HEIGHT-4
    cp (hl)
    ret z
    ld de,yOff
    ld a,(de)
    inc a
    inc a
    and $0F
    ld (de),a
    ret nz      ;if we have no overflow, we're all good.
    inc (hl)
    ret

;##########################
;#UPDATEROTATION
;#After changing the x offset
;# of the map, we need to update
;# the rotation in the fastcopy
;# routine.
;#Explanation:
;#It's a bit confusing i admit,
;# if you have any questions about
;# this part it'd probably be better
;# for now just to ask me personally
;# about it.
;##########################
updateRotation:
    ld a,(xOff)         ;what is the xOffset?
    and 7               ;we're only interested in the lower three bits (essentially xOff % 8)
    ld hl,gbufMask
    ld e,a
    ld d,0
    add hl,de           ;pointer to the rotation mask
    ex af,af'           ;save a (swap with shadow register)
    ld a,(hl)
    ld hl,maskLeft
    ld (hl),a
    ld hl,maskRight
    cpl                 ;xor $FF
    ld (hl),a
    ex af,af'

    ld hl,rotateRight
    cp 4
    jr nc,rotarDer
    ld hl,rotateLeft
rotarDer:
    and %00000011
    ld e,a
    ld d,0
    add hl,de
    push hl
    ld de,rotLeft
    ldi
    ldi
    ldi
    ldi
    pop hl
    ld de,rotRight
    ldi
    ldi
    ldi
    ldi
    ret

rotateRight:       ;if offset greater than or equal to 4, rotate the gbuf right up to four times
    rrca
    rrca
    rrca
    rrca
rotateLeft:       ;the nops keep it smoother (same delay as rlca)
    nop          ; so whether we shift or not, it will take the same amount of cycles
    nop
    nop
    nop
    rlca
    rlca
    rlca
    rlca
gbufMask:
.db %11111111   ;0 display all of the first byte, none of the second
.db %11111110   ;1 after rotating left once, we only want the 7 leftern most bits of byte 1, and rightmost bit of byte 2
.db %11111100   ;2
.db %11111000   ;3
.db %11110000   ;4
.db %11100000   ;5
.db %11000000   ;6
.db %10000000   ;7

drawGbuf:
    ld a,$80
    out ($10),a     ;set row ($80: 0, to $BF:63)
    ld hl,saferam1-14+(14*64)-14    ;
                                    ;
    
xOff = $+1
    ld a,0          ;if xOff > 8, skip the first byte (it's offscreen)
    cp 8
    jr c,$+3
    inc hl
noSkip:
yOff = $+1
    ld b,0
    inc b
    ld de,14
    add hl,de
    djnz $-4
noSkip2:
    ld a,$20        ;$20: col 0, $2E: col 14
    ld c,a
    ld b,64         ;64 filas
fastCopyAgain:
    inc c           ;avanzar a proxima fila
    push bc
    ld de,-(14*64)
    out ($10),a     ;actualizar columna
    add hl,de
    ld de,13
    inc hl
fastCopyLoop:
    add hl,de
    ld a,(hl)       ;cargar valor en gbuf a a
rotLeft:
 .db 0,0,0,0            ;rotate the values we need
maskLeft = $+1
    and $FF         ;necesitamos los valores de la izquierda
    ld c,a
    inc hl
    ld a,(hl)       ;el proximo byte que llena el resto del primero
rotRight:
 .db 0,0,0,0
maskRight = $+1
    and 0
    or c
    out ($11),a
    djnz fastCopyLoop
    pop bc
    ld a,c
    cp $2B+1
    jr nz,fastCopyAgain
    ret

#include "monoMap.asm"
#include "monoSpriteRoutines.asm"
#include "monoMove.asm"
#include "monoText.asm"
#include "monoChance.asm"

;######_DATA_######
#include "monopoly_map.asm"
#include "monopoly_map_tiles.asm"
#include "monopoly_sprites.asm"
#include "monopoly_alphabet.inc"

players:
player1:
p1money:
.dw 0
p1x:
.db 0
p1y:
.db 0
p1Dir:
.db 0
p1char:
.db 0

player2:
p2money:
.dw 0
p2x:
.db 0
p2y:
.db 0
p2Dir:
.db 0
p2char:
.db 0

textLandedOn:
;You've landed on
.db 17,28,32,68,52,23,$FE,53,19,27,22,23,22,$FE,28,27,$FF

textGO:
;You have passed GO, please collect $200.
.db 17,28,32,$FE,26,19,52,23,$FE,29,19,65,65,23,22,$FE,6,10,63,$FE,29,53,23,19,65,23,$FE,21,28,53,53,23,21,55,$FE,60,36,10,10,62,$FF